@@ -35,7 +35,7 @@
 DevicesModel::DevicesModel(QObject *parent)
  : QStandardItemModel(parent),
    m_request(0),
-   m_rx("[a-z]+://?")
+   m_rx("[a-z]+://.*")
 {
     qDBusRegisterMetaType<MapSS>();
     qDBusRegisterMetaType<MapSMapSS>();
@@ -47,6 +47,14 @@ DevicesModel::DevicesModel(QObject *parent)
     m_blacklistedURIs << QLatin1String("scsi");
     m_blacklistedURIs << QLatin1String("http");
     m_blacklistedURIs << QLatin1String("delete");
+
+    // Adds the other device which is meant for manual URI input
+    insertDevice("other",
+                 QString(),
+                 i18nc("@item", "Manual URI"),
+                 QString(),
+                 "other",
+                 QString());
 }
 
 void DevicesModel::update()
@@ -56,22 +64,16 @@ void DevicesModel::update()
     }
 
     // clear the model to don't duplicate items
-    clear();
+    if (rowCount()) {
+        removeRows(1, rowCount() - 1);
+    }
     m_request = new KCupsRequest;
     connect(m_request, SIGNAL(device(QString,QString,QString,QString,QString,QString)),
             this, SLOT(gotDevice(QString,QString,QString,QString,QString,QString)));
     connect(m_request, SIGNAL(finished()), this, SLOT(finished()));
 
     // Get devices with 5 seconds of timeout
-    m_request->getDevices(5);
-
-    // Adds the other device which is meant for manual URI input
-    insertDevice(QString(),
-                 QString(),
-                 i18nc("@item", "Manual URI"),
-                 QString(),
-                 "other",
-                 QString());
+    m_request->getDevices(10);
 }
 
 
@@ -154,6 +156,48 @@ void DevicesModel::insertDevice(const QString &device_class,
                                 const QString &device_uri,
                                 const QString &device_location,
                                 const QStringList &grouped_uris)
+{
+    QStandardItem *stdItem;
+    stdItem = createItem(device_class,
+                         device_id,
+                         device_info,
+                         device_make_and_model,
+                         device_uri,
+                         device_location,
+                         !grouped_uris.isEmpty());
+    if (!grouped_uris.isEmpty()) {
+        stdItem->setData(grouped_uris, DeviceUris);
+    }
+}
+
+void DevicesModel::insertDevice(const QString &device_class,
+                                const QString &device_id,
+                                const QString &device_info,
+                                const QString &device_make_and_model,
+                                const QString &device_uri,
+                                const QString &device_location,
+                                const KCupsPrinters &grouped_printers)
+{
+    QStandardItem *stdItem;
+    stdItem = createItem(device_class,
+                         device_id,
+                         device_info,
+                         device_make_and_model,
+                         device_uri,
+                         device_location,
+                         !grouped_printers.isEmpty());
+    if (!grouped_printers.isEmpty()) {
+        stdItem->setData(qVariantFromValue(grouped_printers), DeviceUris);
+    }
+}
+
+QStandardItem *DevicesModel::createItem(const QString &device_class,
+                                        const QString &device_id,
+                                        const QString &device_info,
+                                        const QString &device_make_and_model,
+                                        const QString &device_uri,
+                                        const QString &device_location,
+                                        bool grouped)
 {
     // "direct"
     kDebug() << device_class;
@@ -178,11 +222,13 @@ void DevicesModel::insertDevice(const QString &device_class,
             // just "http"
             kind = OtherNetworked;
         }
+    } else if (device_class == QLatin1String("other") &&
+               device_uri == QLatin1String("other")) {
+        kind = Other;
     } else {
         // If device class is not network assume local
         kind = Local;
     }
-    kDebug() << device_class << kind << device_uri;
 
     QString location;
     if (device_location.isEmpty() && kind == Local) {
@@ -193,7 +239,7 @@ void DevicesModel::insertDevice(const QString &device_class,
 
     QString text;
     if (!device_make_and_model.isEmpty() &&
-            grouped_uris.isEmpty() &&
+            !grouped &&
             device_make_and_model.compare(QLatin1String("unknown"), Qt::CaseInsensitive)) {
         text = device_info % QLatin1String(" (") % device_make_and_model % QLatin1Char(')');
     } else {
@@ -201,7 +247,7 @@ void DevicesModel::insertDevice(const QString &device_class,
     }
 
     QString toolTip;
-    if (grouped_uris.isEmpty()) {
+    if (!grouped) {
         if (device_uri.startsWith(QLatin1String("parallel"))) {
             toolTip = i18nc("@info:tooltip",
                             "A printer connected to the parallel port");
@@ -236,12 +282,8 @@ void DevicesModel::insertDevice(const QString &device_class,
     stdItem->setData(device_class, DeviceClass);
     stdItem->setData(device_id, DeviceId);
     stdItem->setData(device_info, DeviceInfo);
+    stdItem->setData(device_uri, DeviceUri);
     stdItem->setData(device_make_and_model, DeviceMakeAndModel);
-    if (grouped_uris.isEmpty()) {
-        stdItem->setData(device_uri, DeviceUris);
-    } else {
-        stdItem->setData(grouped_uris, DeviceUris);
-    }
     stdItem->setData(device_location, DeviceLocation);
 
     // Find the proper category to our item
@@ -249,16 +291,21 @@ void DevicesModel::insertDevice(const QString &device_class,
     switch (kind) {
     case Networked:
         catItem = findCreateCategory(i18nc("@item", "Discovered Network Printers"));
+        catItem->appendRow(stdItem);
         break;
     case OtherNetworked:
         catItem = findCreateCategory(i18nc("@item", "Other Network Printers"));
+        catItem->appendRow(stdItem);
+        break;
+    case Local:
+        catItem = findCreateCategory(i18nc("@item", "Local Printers"));
+        catItem->appendRow(stdItem);
         break;
     default:
-        catItem = findCreateCategory(i18nc("@item", "Local Printers"));
+        appendRow(stdItem);
     }
 
-    // Append the devie item to the row
-    catItem->appendRow(stdItem);
+    return stdItem;
 }
 
 void DevicesModel::getGroupedDevicesSuccess(const QDBusMessage &message)
@@ -326,7 +373,7 @@ QStandardItem* DevicesModel::findCreateCategory(const QString &category)
     QFont font = catItem->font();
     font.setBold(true);
     catItem->setFont(font);
-    catItem->setSelectable(false);
+    catItem->setFlags(Qt::ItemIsEnabled);
     appendRow(catItem);
 
     // Emit the parent so the view expand the item
