@@ -21,8 +21,8 @@
 #include "PrintQueueUi.h"
 #include "ui_PrintQueueUi.h"
 
-#include "PrintQueueModel.h"
-#include "PrintQueueSortFilterProxyModel.h"
+#include <JobModel.h>
+#include <JobSortFilterModel.h>
 
 #include <KCupsRequest.h>
 #include <KCupsPrinter.h>
@@ -57,6 +57,7 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
     connect(ui->cancelJobPB, SIGNAL(clicked()), this, SLOT(cancelJob()));
     connect(ui->holdJobPB, SIGNAL(clicked()), this, SLOT(holdJob()));
     connect(ui->resumeJobPB, SIGNAL(clicked()), this, SLOT(resumeJob()));
+    connect(ui->reprintPB, SIGNAL(clicked()), this, SLOT(reprintJob()));
 
     connect(ui->pausePrinterPB, SIGNAL(clicked()), this, SLOT(pausePrinter()));
     connect(ui->configurePrinterPB, SIGNAL(clicked()), this, SLOT(configurePrinter()));
@@ -100,19 +101,21 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
     ui->printerStatusMsgL->setText(QString());
 
     // setup the jobs model
-    m_model = new PrintQueueModel(printer.name(), winId(), this);
+    m_model = new JobModel(this);
+    m_model->setParentWId(winId());
+    m_model->init(printer.name());
     connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(updateButtons()));
     connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(update()));
-    m_proxyModel = new PrintQueueSortFilterProxyModel(this);
+    m_proxyModel = new JobSortFilterModel(this);
     m_proxyModel->setSourceModel(m_model);
     m_proxyModel->setDynamicSortFilter(true);
 
     ui->jobsView->setModel(m_proxyModel);
     ui->jobsView->setItemDelegate(new NoSelectionRectDelegate(this));
     // sort by status column means the jobs will be sorted by the queue order
-    ui->jobsView->sortByColumn(PrintQueueModel::ColStatus, Qt::AscendingOrder);
+    ui->jobsView->sortByColumn(JobModel::ColStatus, Qt::AscendingOrder);
     connect(ui->jobsView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             this, SLOT(updateButtons()));
     connect(ui->jobsView, SIGNAL(customContextMenuRequested(QPoint)),
@@ -124,16 +127,16 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
     QHeaderView *header = ui->jobsView->header();
     header->setResizeMode(QHeaderView::Interactive);
     header->setStretchLastSection(false);
-    header->setResizeMode(PrintQueueModel::ColStatus,        QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColName,          QHeaderView::Stretch);
-    header->setResizeMode(PrintQueueModel::ColUser,          QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColCreated,       QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColCompleted,     QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColPages,         QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColProcessed,     QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColSize,          QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColStatusMessage, QHeaderView::ResizeToContents);
-    header->setResizeMode(PrintQueueModel::ColPrinter,       QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColStatus,        QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColName,          QHeaderView::Stretch);
+    header->setResizeMode(JobModel::ColUser,          QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColCreated,       QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColCompleted,     QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColPages,         QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColProcessed,     QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColSize,          QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColStatusMessage, QHeaderView::ResizeToContents);
+    header->setResizeMode(JobModel::ColPrinter,       QHeaderView::ResizeToContents);
 
     KConfig config("print-manager");
     KConfigGroup printQueue(&config, "PrintQueue");
@@ -142,10 +145,11 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
         header->restoreState(printQueue.readEntry("ColumnState", QByteArray()));
     } else {
         // Hide some columns ColPrinter
-        header->hideSection(PrintQueueModel::ColPrinter);
-        header->hideSection(PrintQueueModel::ColUser);
-        header->hideSection(PrintQueueModel::ColCompleted);
-        header->hideSection(PrintQueueModel::ColSize);
+        header->hideSection(JobModel::ColPrinter);
+        header->hideSection(JobModel::ColUser);
+        header->hideSection(JobModel::ColCompleted);
+        header->hideSection(JobModel::ColSize);
+        header->hideSection(JobModel::ColFromHost);
     }
 
     // This is emitted when a printer is modified
@@ -215,7 +219,7 @@ PrintQueueUi::~PrintQueueUi()
 int PrintQueueUi::columnCount(const QModelIndex &parent) const
 {
     if (!parent.isValid()) {
-        return PrintQueueModel::LastColumn;
+        return JobModel::LastColumn;
     }
     return 0;
 }
@@ -251,7 +255,7 @@ void PrintQueueUi::setState(int state, const QString &message)
                 ui->pausePrinterPB->setIcon(KIcon("media-playback-pause"));
             }
             break;
-        case KCupsPrinter::Stoped:
+        case KCupsPrinter::Stopped:
             m_printerPaused = true;
             ui->statusL->setText(i18n("Printer paused"));
             ui->pausePrinterPB->setText(i18n("Resume Printer"));
@@ -333,7 +337,7 @@ void PrintQueueUi::showContextMenu(const QPoint &point)
                 QAction *action = menu->exec(ui->jobsView->mapToGlobal(point));
                 if (action) {
                     // move the job
-                    modifyJob(PrintQueueModel::Move, action->data().toString());
+                    modifyJob(JobModel::Move, action->data().toString());
                 }
             }
         }
@@ -450,9 +454,9 @@ void PrintQueueUi::update()
 
 void PrintQueueUi::updateButtons()
 {
-    bool cancel, hold, release;
+    bool cancel, hold, release, reprint;
     // Set all options to false
-    cancel = hold = release = false;
+    cancel = hold = release = reprint = false;
 
     QItemSelection selection;
     // we need to map the selection to source to get the real indexes
@@ -461,8 +465,7 @@ void PrintQueueUi::updateButtons()
     if (!selection.indexes().isEmpty()) {
         foreach (const QModelIndex &index, selection.indexes()) {
             if (index.column() == 0) {
-                switch (static_cast<ipp_jstate_t>(index.data(PrintQueueModel::JobState).toInt()))
-                {
+                switch (static_cast<ipp_jstate_t>(index.data(JobModel::RoleJobState).toInt())) {
                     case IPP_JOB_CANCELED :
                     case IPP_JOB_COMPLETED :
                     case IPP_JOB_ABORTED :
@@ -476,6 +479,9 @@ void PrintQueueUi::updateButtons()
                         cancel = hold = true;
                         break;
                 }
+                if (index.data(JobModel::RoleJobRestartEnabled).toBool()) {
+                    reprint = true;
+                }
             }
         }
     }
@@ -483,6 +489,7 @@ void PrintQueueUi::updateButtons()
     ui->cancelJobPB->setEnabled(cancel);
     ui->holdJobPB->setEnabled(hold);
     ui->resumeJobPB->setEnabled(release);
+    ui->reprintPB->setEnabled(reprint);
 }
 
 void PrintQueueUi::modifyJob(int action, const QString &destName)
@@ -495,8 +502,8 @@ void PrintQueueUi::modifyJob(int action, const QString &destName)
         if (index.column() == 0) {
             KCupsRequest *request;
             request = m_model->modifyJob(index.row(),
-                                        static_cast<PrintQueueModel::JobAction>(action),
-                                        destName);
+                                         static_cast<JobModel::JobAction>(action),
+                                         destName);
             if (!request) {
                 // probably the job already has this state
                 // or this is an unknown action
@@ -505,18 +512,21 @@ void PrintQueueUi::modifyJob(int action, const QString &destName)
             request->waitTillFinished();
             if (request->hasError()) {
                 QString msg, jobName;
-                jobName = m_model->item(index.row(), static_cast<int>(PrintQueueModel::ColName))->text();
+                jobName = m_model->item(index.row(), static_cast<int>(JobModel::ColName))->text();
                 switch (action) {
-                case PrintQueueModel::Cancel:
+                case JobModel::Cancel:
                     msg = i18n("Failed to cancel '%1'", jobName);
                     break;
-                case PrintQueueModel::Hold:
+                case JobModel::Hold:
                     msg = i18n("Failed to hold '%1'", jobName);
                     break;
-                case PrintQueueModel::Release:
+                case JobModel::Release:
                     msg = i18n("Failed to release '%1'", jobName);
                     break;
-                case PrintQueueModel::Move:
+                case JobModel::Reprint:
+                    msg = i18n("Failed to reprint '%1'", jobName);
+                    break;
+                case JobModel::Move:
                     msg = i18n("Failed to move '%1' to '%2'", jobName, destName);
                     break;
                 }
@@ -561,35 +571,40 @@ void PrintQueueUi::configurePrinter()
 void PrintQueueUi::cancelJob()
 {
     // CANCEL a job
-    modifyJob(PrintQueueModel::Cancel);
+    modifyJob(JobModel::Cancel);
 }
 
 void PrintQueueUi::holdJob()
 {
     // HOLD a job
-    modifyJob(PrintQueueModel::Hold);
+    modifyJob(JobModel::Hold);
 }
 
 void PrintQueueUi::resumeJob()
 {
     // RESUME a job
-    modifyJob(PrintQueueModel::Release);
+    modifyJob(JobModel::Release);
+}
+
+void PrintQueueUi::reprintJob()
+{
+    modifyJob(JobModel::Reprint);
 }
 
 void PrintQueueUi::whichJobsIndexChanged(int index)
 {
-    int whichJobs;
     switch (index) {
     case 1:
-        whichJobs = CUPS_WHICHJOBS_COMPLETED;
+        m_model->setWhichJobs(JobModel::WhichCompleted);
         break;
     case 2:
-        whichJobs = CUPS_WHICHJOBS_ALL;
+        m_model->setWhichJobs(JobModel::WhichAll);
         break;
     default:
-        whichJobs = CUPS_WHICHJOBS_ACTIVE;
+        m_model->setWhichJobs(JobModel::WhichActive);
+        break;
     }
-    m_model->setWhichJobs(whichJobs);
+
 }
 
 void PrintQueueUi::setupButtons()
@@ -604,7 +619,9 @@ void PrintQueueUi::setupButtons()
 
     // resume job action
     // TODO we need a new icon
-    ui->resumeJobPB->setIcon(KIcon("media-playback-play"));
+    ui->resumeJobPB->setIcon(KIcon("media-playback-start"));
+
+    ui->reprintPB->setIcon(KIcon("view-refresh"));
 
     ui->whichJobsCB->setItemIcon(0, KIcon("view-filter"));
     ui->whichJobsCB->setItemIcon(1, KIcon("view-filter"));
